@@ -1,32 +1,34 @@
 // This file is part of retro-gtk. License: GPL-3.0+.
 
+/**
+ * SECTION:retro-controller
+ * @short_description: An interface representing an input controller
+ * @title: RetroController
+ * @See_also: #retro-controller-codes
+ */
+
 #include "retro-controller.h"
 
 G_DEFINE_INTERFACE (RetroController, retro_controller, G_TYPE_OBJECT);
 
+enum {
+  SIG_STATE_CHANGED_SIGNAL,
+  N_SIGNALS,
+};
+
+static guint signals[N_SIGNALS];
+
 static void
 retro_controller_default_init (RetroControllerInterface *iface)
 {
-}
-
-/**
- * retro_controller_poll:
- * @self: a #RetroController
- *
- * Polls the pending input events for @self.
- */
-void
-retro_controller_poll (RetroController *self)
-{
-  RetroControllerInterface *iface;
-
-  g_return_if_fail (RETRO_IS_CONTROLLER (self));
-
-  iface = RETRO_CONTROLLER_GET_IFACE (self);
-
-  g_return_if_fail (iface->poll != NULL);
-
-  iface->poll (self);
+  signals[SIG_STATE_CHANGED_SIGNAL] =
+    g_signal_new ("state-changed",
+                  G_TYPE_FROM_INTERFACE (iface),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE,
+                  0);
 }
 
 /**
@@ -127,19 +129,15 @@ retro_controller_has_capability (RetroController     *self,
 }
 
 /**
- * retro_controller_set_rumble_state:
+ * retro_controller_get_supports_rumble:
  * @self: a #RetroController
- * @effect: the rumble effect
- * @strength: the rumble effect strength
  *
- * Sets the rumble state of @self.
+ * Gets whether @self supports rumble effects.
  *
- * Returns: whether the rumble state has been successfully set.
+ * Returns: whether @self supports rumble effects.
  */
 gboolean
-retro_controller_set_rumble_state (RetroController   *self,
-                                   RetroRumbleEffect  effect,
-                                   guint16            strength)
+retro_controller_get_supports_rumble (RetroController *self)
 {
   RetroControllerInterface *iface;
 
@@ -147,7 +145,39 @@ retro_controller_set_rumble_state (RetroController   *self,
 
   iface = RETRO_CONTROLLER_GET_IFACE (self);
 
-  g_return_val_if_fail (iface->set_rumble_state != NULL, FALSE);
+  g_return_val_if_fail (iface->get_supports_rumble != NULL, FALSE);
 
-  return iface->set_rumble_state (self, effect, strength);
+  return iface->get_supports_rumble (self);
+}
+
+/**
+ * retro_controller_set_rumble_state:
+ * @self: a #RetroController
+ * @effect: the rumble effect
+ * @strength: the rumble effect strength
+ *
+ * Sets the rumble state of @self.
+ */
+void
+retro_controller_set_rumble_state (RetroController   *self,
+                                   RetroRumbleEffect  effect,
+                                   guint16            strength)
+{
+  RetroControllerInterface *iface;
+
+  g_return_if_fail (RETRO_IS_CONTROLLER (self));
+
+  iface = RETRO_CONTROLLER_GET_IFACE (self);
+
+  g_return_if_fail (iface->set_rumble_state != NULL);
+
+  iface->set_rumble_state (self, effect, strength);
+}
+
+void
+retro_controller_emit_state_changed (RetroController *self)
+{
+  g_return_if_fail (RETRO_IS_CONTROLLER (self));
+
+  g_signal_emit (self, signals[SIG_STATE_CHANGED_SIGNAL], 0);
 }
