@@ -1,21 +1,27 @@
 // This file is part of retro-gtk. License: GPL-3.0+.
 
+/**
+ * SECTION:retro-core-view
+ * @short_description: A widget to handle a core's inputs and outputs
+ * @title: RetroCoreView
+ * @See_also: #RetroCore, #RetroVideoFilter
+ */
+
 #include "retro-core-view.h"
 
-#include "retro-cairo-display-private.h"
 #include "retro-gl-display-private.h"
-#include "retro-controller-codes.h"
+#include "retro-controller-codes-private.h"
 #include "retro-core-view-controller-private.h"
 #include "retro-input-private.h"
 #include "retro-keyboard-private.h"
-#include "retro-pa-player-private.h"
+
+#define RETRO_CONTROLLER_TYPE_COUNT (RETRO_CONTROLLER_TYPE_POINTER + 1)
 
 struct _RetroCoreView
 {
   GtkEventBox parent_instance;
   RetroCore *core;
   RetroGLDisplay *display;
-  RetroPaPlayer *audio_player;
   gboolean can_grab_pointer;
   gboolean snap_pointer_to_borders;
   GHashTable *key_state;
@@ -45,6 +51,13 @@ enum {
 
 static GParamSpec *properties [N_PROPS];
 
+enum {
+  SIG_CONTROLLER_STATE_CHANGED_SIGNAL,
+  N_SIGNALS,
+};
+
+static guint signals [N_SIGNALS];
+
 /* Private */
 
 static void
@@ -103,18 +116,16 @@ recenter_pointer (RetroCoreView *self)
 }
 
 static gboolean
-retro_core_view_get_is_pointer_grabbed (RetroCoreView *self)
+get_is_pointer_grabbed (RetroCoreView *self)
 {
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-
   return self->grabbed_device != NULL;
 }
 
 static void
-retro_core_view_grab (RetroCoreView *self,
-                      GdkDevice     *device,
-                      GdkWindow     *window,
-                      GdkEvent      *event)
+grab (RetroCoreView *self,
+      GdkDevice     *device,
+      GdkWindow     *window,
+      GdkEvent      *event)
 {
   GdkSeat *seat;
   GdkDisplay *display;
@@ -123,10 +134,9 @@ retro_core_view_grab (RetroCoreView *self,
   GdkMonitor *monitor;
   GdkRectangle monitor_geometry;
 
-  g_return_if_fail (RETRO_IS_CORE_VIEW (self));
-  g_return_if_fail (device != NULL);
-  g_return_if_fail (window != NULL);
-  g_return_if_fail (event != NULL);
+  g_assert (device != NULL);
+  g_assert (window != NULL);
+  g_assert (event != NULL);
 
   if (self->grabbed_device != NULL)
     g_object_unref (self->grabbed_device);
@@ -151,16 +161,17 @@ retro_core_view_grab (RetroCoreView *self,
 
   recenter_pointer (self);
 
+  g_signal_emit (self, signals[SIG_CONTROLLER_STATE_CHANGED_SIGNAL], 0);
+
   g_object_unref (cursor);
 }
 
 static void
-retro_core_view_ungrab (RetroCoreView *self)
+ungrab (RetroCoreView *self)
 {
   GdkSeat *seat;
 
-  g_return_if_fail (RETRO_IS_CORE_VIEW (self));
-  g_return_if_fail (self->grabbed_device != NULL);
+  g_assert (self->grabbed_device != NULL);
 
   seat = gdk_device_get_seat (self->grabbed_device);
   gdk_seat_ungrab (seat);
@@ -169,65 +180,61 @@ retro_core_view_ungrab (RetroCoreView *self)
 
   g_clear_object (&self->grabbed_device);
   g_clear_object (&self->grabbed_screen);
+
+  g_signal_emit (self, signals[SIG_CONTROLLER_STATE_CHANGED_SIGNAL], 0);
 }
 
-static gboolean
-retro_core_view_on_key_press_event (GtkWidget   *source,
-                                    GdkEventKey *event,
-                                    gpointer     data)
-{
-  RetroCoreView *self = RETRO_CORE_VIEW (data);
+static gboolean get_key_state (RetroCoreView *self,
+                               guint16        hardware_keycode);
 
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
+static gboolean
+key_press_event_cb (GtkWidget     *source,
+                    GdkEventKey   *event,
+                    RetroCoreView *self)
+{
+  gboolean changed;
 
   if (event->keyval == GDK_KEY_Escape &&
       (event->state & GDK_CONTROL_MASK) &&
-      retro_core_view_get_is_pointer_grabbed (self))
-    retro_core_view_ungrab (self);
+      get_is_pointer_grabbed (self))
+    ungrab (self);
+
+  changed = !get_key_state (self, event->hardware_keycode);
 
   set_input_pressed (self->key_state, event->hardware_keycode);
   set_input_pressed (self->keyval_state, event->keyval);
 
+  if (changed)
+    g_signal_emit (self, signals[SIG_CONTROLLER_STATE_CHANGED_SIGNAL], 0);
+
   return FALSE;
 }
 
 static gboolean
-retro_core_view_on_key_release_event (GtkWidget   *source,
-                                      GdkEventKey *event,
-                                      gpointer     data)
+key_release_event_cb (GtkWidget     *source,
+                      GdkEventKey   *event,
+                      RetroCoreView *self)
 {
-  RetroCoreView *self = RETRO_CORE_VIEW (data);
-
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
   set_input_released (self->key_state, event->hardware_keycode);
   set_input_released (self->keyval_state, event->keyval);
 
+  g_signal_emit (self, signals[SIG_CONTROLLER_STATE_CHANGED_SIGNAL], 0);
+
   return FALSE;
 }
 
 static gboolean
-retro_core_view_on_button_press_event (GtkWidget      *source,
-                                       GdkEventButton *event,
-                                       gpointer        data)
+button_press_event_cb (GtkWidget      *source,
+                       GdkEventButton *event,
+                       RetroCoreView  *self)
 {
-  RetroCoreView *self = RETRO_CORE_VIEW (data);
-
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
   gtk_widget_grab_focus (GTK_WIDGET (source));
 
   if (retro_core_view_get_can_grab_pointer (self)) {
-    if (retro_core_view_get_is_pointer_grabbed (self))
+    if (get_is_pointer_grabbed (self))
       set_input_pressed (self->mouse_button_state, event->button);
     else
-      retro_core_view_grab (self,
-                            event->device,
-                            event->window,
-                            (GdkEvent *) event);
+      grab (self, event->device, event->window, (GdkEvent *) event);
   }
   else {
     set_input_pressed (self->mouse_button_state, event->button);
@@ -239,55 +246,46 @@ retro_core_view_on_button_press_event (GtkWidget      *source,
                                                    &self->pointer_y);
   }
 
+  g_signal_emit (self, signals[SIG_CONTROLLER_STATE_CHANGED_SIGNAL], 0);
+
   return FALSE;
 }
 
 static gboolean
-retro_core_view_on_button_release_event (GtkWidget      *source,
-                                         GdkEventButton *event,
-                                         gpointer        data)
+button_release_event_cb (GtkWidget      *source,
+                         GdkEventButton *event,
+                         RetroCoreView  *self)
 {
-  RetroCoreView *self = RETRO_CORE_VIEW (data);
-
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
   set_input_released (self->mouse_button_state, event->button);
 
+  g_signal_emit (self, signals[SIG_CONTROLLER_STATE_CHANGED_SIGNAL], 0);
+
   return FALSE;
 }
 
 static gboolean
-retro_core_view_on_focus_out_event (GtkWidget     *source,
-                                    GdkEventFocus *event,
-                                    gpointer       data)
+focus_out_event_cb (GtkWidget     *source,
+                    GdkEventFocus *event,
+                    RetroCoreView *self)
 {
-  RetroCoreView *self = RETRO_CORE_VIEW (data);
-
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  if (retro_core_view_get_is_pointer_grabbed (self))
-    retro_core_view_ungrab (self);
+  if (get_is_pointer_grabbed (self))
+    ungrab (self);
 
   reset_input (self->key_state);
   reset_input (self->mouse_button_state);
 
+  g_signal_emit (self, signals[SIG_CONTROLLER_STATE_CHANGED_SIGNAL], 0);
+
   return FALSE;
 }
 
 static gboolean
-retro_core_view_on_motion_notify_event (GtkWidget      *source,
-                                        GdkEventMotion *event,
-                                        gpointer        data)
+motion_notify_event_cb (GtkWidget      *source,
+                        GdkEventMotion *event,
+                        RetroCoreView  *self)
 {
-  RetroCoreView *self = RETRO_CORE_VIEW (data);
-
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
   if (retro_core_view_get_can_grab_pointer (self)) {
-    if (retro_core_view_get_is_pointer_grabbed (self) &&
+    if (get_is_pointer_grabbed (self) &&
         event->device == self->grabbed_device) {
       self->mouse_x_delta += event->x_root - (double) self->screen_center_x;
       self->mouse_y_delta += event->y_root - (double) self->screen_center_y;
@@ -305,51 +303,45 @@ retro_core_view_on_motion_notify_event (GtkWidget      *source,
 
   }
 
+  g_signal_emit (self, signals[SIG_CONTROLLER_STATE_CHANGED_SIGNAL], 0);
+
   return FALSE;
 }
 
 static gboolean
-retro_core_view_get_key_state (RetroCoreView *self,
-                               guint16        hardware_keycode)
+get_key_state (RetroCoreView *self,
+               guint16        hardware_keycode)
 {
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-
   return get_input_state (self->key_state, hardware_keycode);
 }
 
 static gboolean
-retro_core_view_get_joypad_button_state (RetroCoreView *self,
-                                         RetroJoypadId  button)
+get_joypad_button_state (RetroCoreView *self,
+                         RetroJoypadId  button)
 {
   guint16 hardware_keycode;
 
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-
   if (button >= RETRO_JOYPAD_ID_COUNT)
     return 0;
 
   hardware_keycode = retro_key_joypad_mapping_get_button_key (self->key_joypad_mapping, button);
 
-  return retro_core_view_get_key_state (self, hardware_keycode);
+  return get_key_state (self, hardware_keycode);
 }
 
 static gboolean
-retro_core_view_get_mouse_button_state (RetroCoreView *self,
-                                        guint16        button)
+get_mouse_button_state (RetroCoreView *self,
+                        guint16        button)
 {
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-
   return get_input_state (self->mouse_button_state, button);
 }
 
 static gboolean
-retro_core_view_get_keyboard_key_state (RetroCoreView *self,
-                                        guint16        key)
+get_keyboard_key_state (RetroCoreView *self,
+                        guint16        key)
 {
   guint16 keyval;
 
-  g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
-
   if (key >= RETRO_KEYBOARD_KEY_LAST)
     return FALSE;
 
@@ -365,7 +357,6 @@ retro_core_view_finalize (GObject *object)
 
   g_clear_object (&self->core);
   g_object_unref (self->display);
-  g_object_unref (self->audio_player);
   g_hash_table_unref (self->key_state);
   g_hash_table_unref (self->keyval_state);
   g_object_unref (self->key_joypad_mapping);
@@ -458,13 +449,28 @@ retro_core_view_class_init (RetroCoreViewClass *klass)
                                      N_PROPS,
                                      properties);
 
-  gtk_widget_class_set_css_name (widget_class, "retrocoreview");
-}
+  /**
+   * RetroCoreView::controller-state-changed:
+   * @self: the #RetroCoreView
+   *
+   * The ::controller-state-changed signal is emitted when a key is pressed
+   * or released, mouse pointer is moved, or a mouse button is pressed or
+   * released.
+   *
+   * Applications should not connect to it.
+   *
+   * Stability: Private
+   */
+  signals[SIG_CONTROLLER_STATE_CHANGED_SIGNAL] =
+    g_signal_new ("controller-state-changed",
+                  RETRO_TYPE_CORE_VIEW,
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  NULL,
+                  G_TYPE_NONE,
+                  0);
 
-RetroCoreView* retro_core_view_construct (GType object_type) {
-  RetroCoreView * self = NULL;
-  self = (RetroCoreView*) g_object_new (object_type, NULL);
-  return self;
+  gtk_widget_class_set_css_name (widget_class, "retrocoreview");
 }
 
 static void
@@ -483,19 +489,17 @@ retro_core_view_init (RetroCoreView *self)
                           G_BINDING_BIDIRECTIONAL |
                           G_BINDING_SYNC_CREATE);
 
-  self->audio_player = retro_pa_player_new ();
-
   self->key_state = g_hash_table_new_full (g_int_hash, g_int_equal, g_free, g_free);
   self->keyval_state = g_hash_table_new_full (g_int_hash, g_int_equal, g_free, g_free);
   self->key_joypad_mapping = retro_key_joypad_mapping_new_default ();
   self->mouse_button_state = g_hash_table_new_full (g_int_hash, g_int_equal, g_free, g_free);
 
-  g_signal_connect_object (self, "key-press-event", (GCallback) retro_core_view_on_key_press_event, self, 0);
-  g_signal_connect_object (self, "key-release-event", (GCallback) retro_core_view_on_key_release_event, self, 0);
-  g_signal_connect_object (self, "button-press-event", (GCallback) retro_core_view_on_button_press_event, self, 0);
-  g_signal_connect_object (self, "button-release-event", (GCallback) retro_core_view_on_button_release_event, self, 0);
-  g_signal_connect_object (self, "focus-out-event", (GCallback) retro_core_view_on_focus_out_event, self, 0);
-  g_signal_connect_object (self, "motion-notify-event", (GCallback) retro_core_view_on_motion_notify_event, self, 0);
+  g_signal_connect_object (self, "key-press-event", (GCallback) key_press_event_cb, self, 0);
+  g_signal_connect_object (self, "key-release-event", (GCallback) key_release_event_cb, self, 0);
+  g_signal_connect_object (self, "button-press-event", (GCallback) button_press_event_cb, self, 0);
+  g_signal_connect_object (self, "button-release-event", (GCallback) button_release_event_cb, self, 0);
+  g_signal_connect_object (self, "focus-out-event", (GCallback) focus_out_event_cb, self, 0);
+  g_signal_connect_object (self, "motion-notify-event", (GCallback) motion_notify_event_cb, self, 0);
 }
 
 /* Public */
@@ -519,13 +523,11 @@ retro_core_view_set_core (RetroCoreView *self,
   if (self->core != NULL) {
     g_clear_object (&self->core);
     retro_gl_display_set_core (self->display, NULL);
-    retro_pa_player_set_core (self->audio_player, NULL);
   }
 
   if (core != NULL) {
     self->core = g_object_ref (core);
     retro_gl_display_set_core (self->display, core);
-    retro_pa_player_set_core (self->audio_player, core);
   }
 }
 
@@ -695,7 +697,7 @@ retro_core_view_get_input_state (RetroCoreView *self,
     if (!retro_input_get_joypad (input, &id))
       return 0;
 
-    return retro_core_view_get_joypad_button_state (self, id) ? G_MAXINT16 : 0;
+    return get_joypad_button_state (self, id) ? G_MAXINT16 : 0;
   case RETRO_CONTROLLER_TYPE_MOUSE:
     if (!retro_input_get_mouse (input, &id))
       return 0;
@@ -712,9 +714,9 @@ retro_core_view_get_input_state (RetroCoreView *self,
 
       return result;
     case RETRO_MOUSE_ID_LEFT:
-      return retro_core_view_get_mouse_button_state (self, 1) ? G_MAXINT16 : 0;
+      return get_mouse_button_state (self, 1) ? G_MAXINT16 : 0;
     case RETRO_MOUSE_ID_RIGHT:
-      return retro_core_view_get_mouse_button_state (self, 3) ? G_MAXINT16 : 0;
+      return get_mouse_button_state (self, 3) ? G_MAXINT16 : 0;
     default:
       return 0;
     }
@@ -722,7 +724,7 @@ retro_core_view_get_input_state (RetroCoreView *self,
     if (!retro_input_get_keyboard (input, &id))
       return 0;
 
-    return retro_core_view_get_keyboard_key_state (self, id) ? G_MAXINT16 : 0;
+    return get_keyboard_key_state (self, id) ? G_MAXINT16 : 0;
   case RETRO_CONTROLLER_TYPE_POINTER:
     if (!retro_input_get_pointer (input, &id))
       return 0;
@@ -737,7 +739,7 @@ retro_core_view_get_input_state (RetroCoreView *self,
           retro_core_view_get_snap_pointer_to_borders (self))
         return 0;
 
-      return retro_core_view_get_mouse_button_state (self, 1) ? 1 : 0;
+      return get_mouse_button_state (self, 1) ? 1 : 0;
     default:
       return 0;
     }
@@ -810,8 +812,8 @@ retro_core_view_set_can_grab_pointer (RetroCoreView *self,
   self->can_grab_pointer = can_grab_pointer;
 
   if (can_grab_pointer == FALSE &&
-      retro_core_view_get_is_pointer_grabbed (self))
-    retro_core_view_ungrab (self);
+      get_is_pointer_grabbed (self))
+    ungrab (self);
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_CAN_GRAB_POINTER]);
 }
